/******************************************************************************
*
* project name: Shuffle Effect Demo
* file name:    shuffle.c
* initial date: 02/11/2000
* authors:      thomas.nussbaumer@gmx.net
*               Lionel Debroux (xdanger1@caramail.com)
*
* description:  splits homescreen into tiles and shuffles them
*
* !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
* NOTE: current implementation uses TIGCC 0.93
* !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
*
* for TI89 compile with:   tigcc -O3 -DUSE_TI89  dissolve.c
* for TI92P compile with:  tigcc -O3 -DUSE_TI92P dissolve.c
*
*
*
* -----------------------------------------------------------------------------
* Revision History
* -----------------------------------------------------------------------------
*
* v1.2: 09/02/2002
*       Update for TIGCC 0.93, a few improvements and optimizations by
*       Lionel Debroux
*
* v1.0: 04/11/2000
*       Initial version
*
******************************************************************************/

#if defined(USE_TI89) && defined(USE_TI92P)
#error cannot produce both versions at once
#elif defined(USE_TI89)
#define C89_92(x,y) (x)
#elif defined(USE_TI92P)
#define C89_92(x,y) (y)
#else
#error Please specify either -DUSE_TI89 or -DUSE_TI92P on commandline
#endif

#define OPTIMIZE_ROM_CALLS
#define NO_EXIT_SUPPORT

#include <tigcclib.h>

#define MAX_TILESIZE 5

#define sizex   5
#define sizey   C89_92(5,4)
#define nrcols  C89_92(32,48)
#define nrlines C89_92(20,32)

SCR_RECT fullscreen={{0,0,239,127}};

typedef struct {
    unsigned short NumRows, NumCols;
    unsigned char  Data[MAX_TILESIZE*MAX_TILESIZE];  // maybe this is too large, but I don't care
} MYBITMAP;


//=============================================================================
// swap two tiles using source as screenbuffer
//=============================================================================
static void SwapFields(short field1,short field2) {
    MYBITMAP bm1;
    MYBITMAP bm2;
    SCR_RECT rect1;
    SCR_RECT rect2;

    //------------------------------
    // set size of BITMAP structures
    //------------------------------
    bm1.NumRows = bm2.NumRows = sizex;
    bm1.NumCols = bm2.NumCols = sizey;

    //----------------------------------------------------
    // fill up the rectangles and get the fill the bitmaps
    //----------------------------------------------------
    rect1.xy.x0=(field1 % nrcols)*sizex;
    rect1.xy.y0=(field1 / nrcols)*sizey;
    rect1.xy.x1=rect1.xy.x0+sizex-1;
    rect1.xy.y1=rect1.xy.y0+sizey-1;

    rect2.xy.x0=(field2 % nrcols)*sizex;
    rect2.xy.y0=(field2 / nrcols)*sizey;
    rect2.xy.x1=rect2.xy.x0+sizex-1;
    rect2.xy.y1=rect2.xy.y0+sizey-1;

    BitmapGet(&rect1,(BITMAP*)&bm1);
    BitmapGet(&rect2,(BITMAP*)&bm2);

    //--------------------------------------------------------------
    // put the bitmaps in switched order into the given screenbuffer
    //--------------------------------------------------------------
    BitmapPut(rect1.xy.x0,rect1.xy.y0, (BITMAP*)&bm2, &fullscreen, A_REPLACE);
    BitmapPut(rect2.xy.x0,rect2.xy.y0, (BITMAP*)&bm1, &fullscreen, A_REPLACE);
}


//=============================================================================
// executes the screen tile shuffle
//=============================================================================
static void Shuffle(void *buf) {
    short i,j,k;
    short key;
    register void *kbq = kbd_queue();

    //------------------------------
    // get original screen content
    //------------------------------
    LCD_save(buf - LCD_SIZE);
    LCD_save(buf);

    begin:
    LCD_restore(buf - LCD_SIZE);
    LCD_save(buf);
    randomize();

    //-----------------------------------------------------------------------
    // tell the AMS that we are using our own screen buffer for manipulations
    //-----------------------------------------------------------------------
    PortSet(buf,239,127);

    //-------------------------------------
    // swap tiles until user gets bored ...
    //-------------------------------------
    while (1) {
        i = random(nrlines); // get a random line number
        j = random(nrcols);  // get a random column number
        k = random(4);       // get a random direction to switch

        if (k==0) {
            //------------------------------
            // switch with tile above
            //------------------------------
            if (!i) i++;
            SwapFields(/*i,j,i-1,j)*/i*nrcols+j,(i-1)*nrcols+j);
        }
        else if (k==1) {
            //------------------------------
            // switch with tile below
            //------------------------------
            if (i==nrlines-1) i--;
            SwapFields(/*i,j,i+1,j)*/i*nrcols+j,(i+1)*nrcols+j);
        }
        else if (k==2) {
            //------------------------------
            // switch with tile to the left
            //------------------------------
            if (!j) j++;
            SwapFields(/*i,j,i,j-1)*/i*nrcols+j,i*nrcols+j-1);
        }
        else if (k==3) {
            //------------------------------
            // switch with tile to the right
            //------------------------------
            if (j==nrcols-1) j--;
            SwapFields(/*i,j,i,j+1)*/i*nrcols+j,i*nrcols+j+1);
        }
        //-----------------------------
        // draw modified screen content
        //-----------------------------
        LCD_restore(buf);

        if (!OSdequeue(&key,kbq)) {
            if (key == KEY_CLEAR) goto begin;
            else break;
        }
    }
    //----------------------------------------------------
    // clear keyboard buffer
    //----------------------------------------------------
    GKeyFlush();

    PortRestore();
}


//=============================================================================
// where all the fun starts as usual ...
//=============================================================================
void _main(void) {
    void *savescreen = NULL;

    if (!(savescreen = malloc(LCD_SIZE+LCD_SIZE+LCD_SIZE))) {
        ST_helpMsg("ERROR: Not enough memory");
        return;
    }

    LCD_save(savescreen);
    Shuffle(savescreen + LCD_SIZE + LCD_SIZE);
    LCD_restore(savescreen);

    free(savescreen);

    ST_helpMsg("TICT Production 2002");
}

